package com.ruoyi.project.VRdemo.controller;

import java.io.IOException;
import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.excel.EasyExcel;
import com.ruoyi.common.utils.poi.StudyImListener;
import com.ruoyi.framework.aspectj.lang.annotation.Anonymous;
import com.ruoyi.project.VRdemo.domain.StudyImData;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.VRdemo.domain.Press;
import com.ruoyi.project.VRdemo.service.IPressService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;
import org.springframework.web.multipart.MultipartFile;

/**
 * pressController
 *
 * @author ruoyi
 * @date 2024-01-19
 */
@RestController
@RequestMapping("/vr/press")
public class PressController extends BaseController {
    @Autowired
    private IPressService pressService;

    /**
     * 查询press列表
     */
    @PreAuthorize("@ss.hasPermi('vr:press:list')")
    @GetMapping("/list")
    public TableDataInfo list(Press press) {
        startPage();
        List<Press> list = pressService.selectPressList(press);
        return getDataTable(list);
    }

    /**
     * 导出press列表
     */
    @PreAuthorize("@ss.hasPermi('vr:press:export')")
    @Log(title = "press", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, Press press) {
        List<Press> list = pressService.selectPressList(press);
        ExcelUtil<Press> util = new ExcelUtil<Press>(Press.class);
        util.exportExcel(response, list, "press数据");
    }

    /**
     * 获取press详细信息
     */
    @PreAuthorize("@ss.hasPermi('vr:press:query')")
    @GetMapping(value = "/{pressId}")
    public AjaxResult getInfo(@PathVariable("pressId") Long pressId) {
        return success(pressService.selectPressByPressId(pressId));
    }

    /**
     * 新增press
     */
    @PreAuthorize("@ss.hasPermi('vr:press:add')")
    @Log(title = "press", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody Press press) {
        return toAjax(pressService.insertPress(press));
    }

    /**
     * 修改press
     */
    @PreAuthorize("@ss.hasPermi('vr:press:edit')")
    @Log(title = "press", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody Press press) {
        return toAjax(pressService.updatePress(press));
    }

    /**
     * 删除press
     */
    @PreAuthorize("@ss.hasPermi('vr:press:remove')")
    @Log(title = "press", businessType = BusinessType.DELETE)
    @DeleteMapping("/{pressIds}")
    public AjaxResult remove(@PathVariable Long[] pressIds) {
        return toAjax(pressService.deletePressByPressIds(pressIds));
    }
}
