package com.ruoyi.project.VRdemo.controller;

import cn.dev33.satoken.stp.StpUtil;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.FileUploadUtils;
import com.ruoyi.common.utils.file.FileUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Anonymous;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.framework.config.RuoYiConfig;
import com.ruoyi.framework.config.ServerConfig;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.framework.web.page.TableDataInfo;
import com.ruoyi.project.VRdemo.domain.NoteNationData;
import com.ruoyi.project.VRdemo.domain.StudyData;
import com.ruoyi.project.VRdemo.domain.vo.Catalog;
import com.ruoyi.project.VRdemo.domain.vo.StudyDataVo;
import com.ruoyi.project.VRdemo.service.IStudyDataService;
import com.ruoyi.project.system.domain.SysDictData;
import com.ruoyi.project.system.service.ISysDictDataService;
import org.apache.commons.io.FilenameUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

@RestController
@RequestMapping("/system/study")
public class StudyDataController extends BaseController {
    @Autowired
    private IStudyDataService studyDataService;

    @Autowired
    private ISysDictDataService sysDictDataService;
    @Autowired
    private ServerConfig serverConfig;

    /**
     * 查询study列表
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @GetMapping("/list")
    public TableDataInfo list(StudyData studyData) {
        Long companyId = getLoginUser().getCompanyId();
        studyData.setCompanyId(companyId);
        startPage();
        List<StudyData> list = studyDataService.selectStudyDataList(studyData);
        return getDataTable(list);
    }

    @Anonymous
    @PostMapping("/studyWithoutNation")
    public AjaxResult LoadingApp(@RequestBody JSONObject jsonObject) {
        Long companyAppId = Long.parseLong(jsonObject.getString("company_app_id"));
        StpUtil.login(companyAppId);

        StudyDataVo studyDataVo = new StudyDataVo();
        studyDataVo.setCompanyId(companyAppId);

        if (jsonObject.getString("long_title").isEmpty()) {
            List<StudyData> list = studyDataService.selectStudyDataListVoAll(studyDataVo);
            if (list == null || list.size() == 0) {
                AjaxResult.error("获取数据失败");
            } else {
                AjaxResult ajax = AjaxResult.success();
                ajax.put("data", list);
                return ajax;
            }
        }
        if (!jsonObject.getString("long_title").isEmpty() || jsonObject.getString("long_title") != "") {
            String longTitle = jsonObject.getString("long_title");
            studyDataVo.setLongTitle(longTitle);
            List<StudyData> list = studyDataService.selectStudyDataListVo(studyDataVo);
            if (list == null || list.size() == 0) {
                AjaxResult.error("获取数据失败");
            } else {
                AjaxResult ajax = AjaxResult.success();
                ajax.put("data", list);
                return ajax;
            }
        }

        return AjaxResult.error("获取数据失败");
    }

    @Anonymous
    @PostMapping("/getCatalog")
    public AjaxResult getCatalog(@RequestBody JSONObject jsonObject) {
        Long companyAppId = Long.parseLong(jsonObject.getString("company_app_id"));
        StpUtil.login(companyAppId);

        StudyDataVo studyDataVo = new StudyDataVo();
        studyDataVo.setCompanyId(companyAppId);

        Catalog catalog = new Catalog();
        catalog.setCompanyId(companyAppId);
        List<Catalog> catalogList = studyDataService.selectCatalog(catalog);
        if (catalogList == null || catalogList.size() == 0) {
            AjaxResult.error("获取数据失败");
        } else {
            AjaxResult ajax = AjaxResult.success();
            ajax.put("catalog", catalogList);
            return ajax;
        }

        return AjaxResult.error("获取数据失败");
    }

    @Anonymous
    @PostMapping("/getTimeData")
    public AjaxResult getTimeData(@RequestBody JSONObject jsonObject) {
        Long companyAppId = Long.parseLong(jsonObject.getString("company_app_id"));

        StpUtil.login(companyAppId);
        SysDictData sysDictDataMonth = new SysDictData();
        sysDictDataMonth.setDictType("month_type");
        List<SysDictData> monthlist = sysDictDataService.selectDictDataList(sysDictDataMonth);
        if (monthlist == null || monthlist.size() == 0) {
            AjaxResult.error("获取数据失败");
        }
        SysDictData sysDictDataYear = new SysDictData();
        sysDictDataYear.setDictType("year_type");
        List<SysDictData> yearlist = sysDictDataService.selectDictDataList(sysDictDataYear);
        if (monthlist == null || monthlist.size() == 0) {
            AjaxResult.error("获取数据失败");
        }

        AjaxResult ajax = AjaxResult.success();
        ajax.put("month", monthlist);
        ajax.put("year", yearlist);
        return ajax;
    }


    /**
     * 导出study列表
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @Log(title = "study", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, StudyData studyData) {
        List<StudyData> list = studyDataService.selectStudyDataList(studyData);
        ExcelUtil<StudyData> util = new ExcelUtil<StudyData>(StudyData.class);
        util.exportExcel(response, list, "study数据");
    }

    /**
     * 获取study详细信息
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(studyDataService.selectStudyDataById(id));
    }

    /**
     * 新增study
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @Log(title = "study", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody StudyData studyData) {
        //获取编号为00的文件路径
        String pathStr = studyData.getPath();
        if(StringUtils.isNull(pathStr)){
            return error("上传文件不能为空");
        }
        String[] paths = pathStr.split(","); //将批量文件名转化为数组
        String zero = "";
        for (String s : paths) {
            String fileName = s.substring(s.lastIndexOf("/") + 1, s.lastIndexOf(".")); //获取文件名
            String last = fileName.substring(fileName.length() - 2); //获取文件名后面的编号
            if (last.equals("00")) {
                zero = s;
            }
        }
        studyData.setPath(zero);
        int count = paths.length;//获取上传文件数量
        studyData.setCount((long) count);

        if (!studyDataService.checkUserNameUnique(studyData)) {
            return error("新增标题'" + studyData.getTitle() + "'失败，该标题已存在");
        }
        Long companyId = getLoginUser().getCompanyId();
        studyData.setCompanyId(companyId);
        studyData.setDataType(2L);

//        SysDictData sysDictDataMonth = new SysDictData();
//        sysDictDataMonth.setDictValue(studyData.getMonth());
//        sysDictDataMonth.setDictType("month_type");
//        List<SysDictData> monthlist = sysDictDataService.selectDictDataList(sysDictDataMonth);
//
//        SysDictData sysDictDataYear = new SysDictData();
//        sysDictDataYear.setDictValue(studyData.getYear());
//        sysDictDataYear.setDictType("year_type");
//        List<SysDictData> yearlist = sysDictDataService.selectDictDataList(sysDictDataYear);
//        studyData.setLongTitle(yearlist.get(0).getDictLabel().toString()+monthlist.get(0).getDictLabel().toString()+studyData.getTitle());

        return toAjax(studyDataService.insertStudyData(studyData));
    }

    /**
     * 通用上传请求（单个）
     */
    @PostMapping("/upload")
    public AjaxResult uploadFile(MultipartFile file) throws Exception {
        try {
            String extension = FilenameUtils.getExtension(file.getOriginalFilename());
            String filePath;

            if (extension.equalsIgnoreCase("pdf")) {
                filePath = RuoYiConfig.getuploadPdf();
            } else if (extension.equalsIgnoreCase("doc") || extension.equalsIgnoreCase("docx")) {
                filePath = RuoYiConfig.getuploadWord();
            } else {
                filePath = RuoYiConfig.getUploadPath();
            }

            String fileName = FileUploadUtils.upload(filePath, file);
            String url = serverConfig.getUrl() + fileName;

            AjaxResult ajax = AjaxResult.success();
            ajax.put("url", url);
            ajax.put("fileName", fileName);
            ajax.put("newFileName", FileUtils.getName(fileName));
            ajax.put("originalFilename", file.getOriginalFilename());

            StudyData studyData = new StudyData();
            studyData.setPath(url);
            studyDataService.updateStudyData(studyData);
            return ajax;
        } catch (Exception e) {
            return AjaxResult.error(e.getMessage());
        }
    }

    @Anonymous
    @PostMapping("/addStudy")
    public AjaxResult addStudy(@RequestBody StudyData studyData) {
//        Long companyId =getLoginUser().getCompanyId();
        studyData.setCompanyId(3L);
        studyData.setDataType(2L);

        return toAjax(studyDataService.insertStudyData(studyData));
    }

    /**
     * 修改study
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @Log(title = "study", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody StudyData studyData) {
        return toAjax(studyDataService.updateStudyData(studyData));
    }


    //删除
    @DeleteMapping("/{ids}")
    public String deleteFile(@PathVariable("ids") String ids) {
        try {
            Long Ids = Long.parseLong(ids);
            System.out.println(Ids);
            boolean success = studyDataService.deleteFile(Ids);
            if (success) {
                return "文件删除成功.";
            } else {
                return "文件删除失败.";
            }
        } catch (NumberFormatException e) {
            return "文件格式错误";
        }
    }
}
