package com.ruoyi.project.VRdemo.controller;

import cn.dev33.satoken.stp.StpUtil;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.FileUploadUtils;
import com.ruoyi.common.utils.file.FileUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Anonymous;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.framework.config.RuoYiConfig;
import com.ruoyi.framework.config.ServerConfig;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.framework.web.page.TableDataInfo;
import com.ruoyi.project.VRdemo.domain.StudyNationData;
import com.ruoyi.project.VRdemo.domain.vo.StudyNationDataVo;
import com.ruoyi.project.VRdemo.service.IStudyNationDataService;
import com.ruoyi.project.system.service.ISysDictDataService;
import org.apache.commons.io.FilenameUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

@RestController
@RequestMapping("/system/studyNation")
public class StudyNationDataController extends BaseController {
    @Autowired
    private IStudyNationDataService studyNationDataService;

    @Autowired
    private ISysDictDataService sysDictDataService;
    @Autowired
    private ServerConfig serverConfig;

    /**
     * 查询studyNation列表
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @GetMapping("/list")
    public TableDataInfo list(StudyNationData studyNationData) {
        Long companyId = getLoginUser().getCompanyId();
        studyNationData.setCompanyId(companyId);

        startPage();
        List<StudyNationData> list = studyNationDataService.selectStudyNationDataList(studyNationData);
        return getDataTable(list);
    }

    @Anonymous
    @PostMapping("/studyWithNation")
    public AjaxResult LoadingApp(@RequestBody JSONObject jsonObject) {
        Long companyAppId = Long.parseLong(jsonObject.getString("company_app_id"));
        StpUtil.login(companyAppId);

        StudyNationDataVo studyNationDataVo = new StudyNationDataVo();
        studyNationDataVo.setCompanyId(companyAppId);

        String searchType = jsonObject.getString("search_type");
        String key = jsonObject.getString("key");

        List<StudyNationData> list;
        if ("1".equals(searchType)) { // 搜索题名
            studyNationDataVo.setLongTitle(key);
            list = studyNationDataService.selectStudyNationDataListVo(studyNationDataVo);
        } else if ("2".equals(searchType)) { // 搜索国名
            studyNationDataVo.setNation(key);
            list = studyNationDataService.selectStudyNationDataListVo(studyNationDataVo);
        } else if ("3".equals(searchType)) { // 搜索年份
            studyNationDataVo.setYear(key);
            list = studyNationDataService.selectStudyNationDataListVo(studyNationDataVo);
        } else {
            return AjaxResult.error("无效的search_type");
        }

        if (list == null || list.isEmpty()) {
            return AjaxResult.error("获取数据失败");
        } else {
            AjaxResult ajax = AjaxResult.success();
            ajax.put("data", list);
            ajax.put("company_app_id", companyAppId);
            ajax.put("search_type", searchType);
            ajax.put("key", key);
            return ajax;
        }

    }

    /**
     * 导出studyNation列表
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @Log(title = "studyNation", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, StudyNationData studyNationData) {
        List<StudyNationData> list = studyNationDataService.selectStudyNationDataList(studyNationData);
        ExcelUtil<StudyNationData> util = new ExcelUtil<StudyNationData>(StudyNationData.class);
        util.exportExcel(response, list, "studyNation数据");
    }

    /**
     * 获取studyNation详细信息
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(studyNationDataService.selectStudyNationDataById(id));
    }

    /**
     * 新增studyNation
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @Log(title = "studyNation", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody StudyNationData studyNationData) {
        //获取编号为00的文件路径
        String pathStr = studyNationData.getFilePath();
        if(StringUtils.isNull(pathStr)){
            return error("上传文件不能为空");
        }
        String[] paths = pathStr.split(","); //将批量文件名转化为数组
        String zero = "";
        for (String s : paths) {
            String fileName = s.substring(s.lastIndexOf("/") + 1, s.lastIndexOf(".")); //获取文件名
            String last = fileName.substring(fileName.length() - 2); //获取文件名后面的编号
            if (last.equals("00")) {
                zero = s;
            }
        }
        studyNationData.setFilePath(zero);
        int count = paths.length;//获取上传文件数量
        studyNationData.setCount((long) count);

        if (!studyNationDataService.checkUserNameUnique(studyNationData)) {
            return error("新增标题'" + studyNationData.getTitle() + "'失败，该标题已存在");
        }
        Long companyId = getLoginUser().getCompanyId();
        studyNationData.setCompanyId(companyId);
        studyNationData.setDataType(3L);

//        SysDictData sysDictDataMonth = new SysDictData();
//        sysDictDataMonth.setDictValue(studyNationData.getMonth());
//        sysDictDataMonth.setDictType("month_type");
//        List<SysDictData> monthlist = sysDictDataService.selectDictDataList(sysDictDataMonth);
//
//        SysDictData sysDictDataYear = new SysDictData();
//        sysDictDataYear.setDictValue(studyNationData.getYear());
//        sysDictDataYear.setDictType("year_type");
//        List<SysDictData> yearlist = sysDictDataService.selectDictDataList(sysDictDataYear);
//        if (studyNationData.getYear()== null && studyNationData.getMonth()== null ) {
//            studyNationData.setLongTitle(studyNationData.getTitle());
//        } else {
//            studyNationData.setLongTitle(yearlist.get(0).getDictLabel().toString() + monthlist.get(0).getDictLabel().toString() + studyNationData.getTitle());
//        }
        return toAjax(studyNationDataService.insertStudyNationData(studyNationData));
    }

    /**
     * 通用上传请求（单个）
     */
    @PostMapping("/upload")
    public AjaxResult uploadFile(MultipartFile file) throws Exception {
        try {
            String extension = FilenameUtils.getExtension(file.getOriginalFilename());
            String filePath;

            if (extension.equalsIgnoreCase("pdf")) {
                filePath = RuoYiConfig.getuploadPdf();
            } else if (extension.equalsIgnoreCase("doc") || extension.equalsIgnoreCase("docx")) {
                filePath = RuoYiConfig.getuploadWord();
            } else {
                filePath = RuoYiConfig.getUploadPath();
            }

            String fileName = FileUploadUtils.upload(filePath, file);
            String url = serverConfig.getUrl() + fileName;

            AjaxResult ajax = AjaxResult.success();
            ajax.put("url", url);
            ajax.put("fileName", fileName);
            ajax.put("newFileName", FileUtils.getName(fileName));
            ajax.put("originalFilename", file.getOriginalFilename());

            StudyNationData studyNationData = new StudyNationData();
            studyNationData.setCoverPath(url);
            studyNationDataService.updateStudyNationData(studyNationData);
            return ajax;
        } catch (Exception e) {
            return AjaxResult.error(e.getMessage());
        }
    }

    @Anonymous
    @PostMapping("addStudyNation")
    public AjaxResult addStudyNation(@RequestBody StudyNationData studyNationData) {
//        Long companyId =getLoginUser().getCompanyId();
        studyNationData.setCompanyId(3L);
        studyNationData.setDataType(3L);

        return toAjax(studyNationDataService.insertStudyNationData(studyNationData));
    }

    /**
     * 修改studyNation
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,xmg')")
    @Log(title = "studyNation", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody StudyNationData studyNationData) {
        return toAjax(studyNationDataService.updateStudyNationData(studyNationData));
    }

    //删除
    @DeleteMapping("/{ids}")
    public String deleteFile(@PathVariable("ids") String ids) {
        try {
            Long Ids = Long.parseLong(ids);
            System.out.println(Ids);
            boolean success = studyNationDataService.deleteFile(Ids);
            if (success) {
                return "文件删除成功.";
            } else {
                return "文件删除失败.";
            }
        } catch (NumberFormatException e) {
            return "文件格式错误";
        }
    }
}
