package com.ruoyi.project.VRdemo.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.VRdemo.domain.Thesis;
import com.ruoyi.project.VRdemo.service.IThesisService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;

/**
 * 论文Controller
 * 
 * @author ruoyi
 * @date 2024-06-23
 */
@RestController
@RequestMapping("/VRdemo/thesis")
public class ThesisController extends BaseController
{
    @Autowired
    private IThesisService thesisService;

    /**
     * 查询论文列表
     */
    @PreAuthorize("@ss.hasPermi('VRdemo:thesis:list')")
    @GetMapping("/list")
    public TableDataInfo list(Thesis thesis)
    {
        startPage();
        List<Thesis> list = thesisService.selectThesisList(thesis);
        return getDataTable(list);
    }

    /**
     * 导出论文列表
     */
    @PreAuthorize("@ss.hasPermi('VRdemo:thesis:export')")
    @Log(title = "论文", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, Thesis thesis)
    {
        List<Thesis> list = thesisService.selectThesisList(thesis);
        ExcelUtil<Thesis> util = new ExcelUtil<Thesis>(Thesis.class);
        util.exportExcel(response, list, "论文数据");
    }

    /**
     * 获取论文详细信息
     */
    @PreAuthorize("@ss.hasPermi('VRdemo:thesis:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(thesisService.selectThesisById(id));
    }

    /**
     * 新增论文
     */
    @PreAuthorize("@ss.hasPermi('VRdemo:thesis:add')")
    @Log(title = "论文", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody Thesis thesis)
    {
        return toAjax(thesisService.insertThesis(thesis));
    }

    /**
     * 修改论文
     */
    @PreAuthorize("@ss.hasPermi('VRdemo:thesis:edit')")
    @Log(title = "论文", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody Thesis thesis)
    {
        return toAjax(thesisService.updateThesis(thesis));
    }

    /**
     * 删除论文
     */
    @PreAuthorize("@ss.hasPermi('VRdemo:thesis:remove')")
    @Log(title = "论文", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(thesisService.deleteThesisByIds(ids));
    }
}
