package com.ruoyi.project.VRdemo.service.impl;

import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.FileUtils;
import com.ruoyi.framework.config.RuoYiConfig;
import com.ruoyi.project.VRdemo.domain.FileUpload;
import com.ruoyi.project.VRdemo.domain.NoteData;
import com.ruoyi.project.VRdemo.domain.vo.NoteDataVo;
import com.ruoyi.project.VRdemo.mapper.NoteDataMapper;
import com.ruoyi.project.VRdemo.service.INoteDataService;
import com.ruoyi.project.system.domain.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.List;

/**
 * @ClassName NoteDataServiceImpl
 * @Description TODO
 * @Author zhixian li
 * @Date 2023年07月17日 16:32
 * @Version 1.0
 */
@Service
public class NoteDataServiceImpl implements INoteDataService {
    @Autowired
    private NoteDataMapper noteDataMapper;

    /**
     * 查询文档管理
     *
     * @param id 文档管理主键
     * @return 文档管理
     */
    @Override
    public NoteData selectNoteDataById(Long id) {
        return noteDataMapper.selectNoteDataById(id);
    }

    /**
     * 查询文档管理列表
     *
     * @param noteData 文档管理
     * @return 文档管理
     */
    @Override
    public List<NoteData> selectNoteDataList(NoteData noteData) {
        return noteDataMapper.selectNoteDataList(noteData);
    }

    /**
     * 新增文档管理
     *
     * @param noteData 文档管理
     * @return 结果
     */
    @Override
    public int insertNoteData(NoteData noteData) {
        noteData.setCreateTime(DateUtils.getNowDate());
        return noteDataMapper.insertNoteData(noteData);
    }

    /**
     * 修改文档管理
     *
     * @param noteData 文档管理
     * @return 结果
     */
    @Override
    public int updateNoteData(NoteData noteData) {
        noteData.setUpdateTime(DateUtils.getNowDate());
        return noteDataMapper.updateNoteData(noteData);
    }

    /**
     * 批量删除文档管理
     *
     * @param ids 需要删除的文档管理主键
     * @return 结果
     */
    @Override
    public int deleteNoteDataByIds(Long[] ids) {
        return noteDataMapper.deleteNoteDataByIds(ids);
    }

    /**
     * 删除文档管理信息
     *
     * @param id 文档管理主键
     * @return 结果
     */
    @Override
    public int deleteNoteDataById(Long id) {
        return noteDataMapper.deleteNoteDataById(id);
    }

    /**
     * 校验用户名称是否唯一
     *
     * @param noteData 用户信息
     * @return 结果
     */
    @Override
    public boolean checkUserNameUnique(NoteData noteData) {
        Long userId = StringUtils.isNull(noteData.getId()) ? -1L : noteData.getId();
        NoteData info = noteDataMapper.checkUserNameUnique(noteData.getTitle());
        if (StringUtils.isNotNull(info) && info.getId().longValue() != userId.longValue()) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    @Override
    public List<NoteData> selectNoteDataListVo(NoteDataVo noteDataVo) {
        return noteDataMapper.selectNoteDataListVo(noteDataVo);
    }

    //删除
    public boolean deleteFile(Long ids) {
        // 根据ID查询文件信息
        NoteData file = noteDataMapper.selectNoteDataById(ids);
        System.out.println("4." + file);
        if (file == null) {
            return false; // 文件不存在，删除失败
        }

        String oldPath = file.getPath();

        System.out.println("oldPath  "+oldPath);

        if (oldPath != null) {
            String filePath = RuoYiConfig.getUploadPath();

            System.out.println("filePath1  "+filePath);

            //由于在RuoYiConfig.getUploadPath()中带有upload导致文件搜索失败，所以去除
            String keyword = "/upload";
            int endIndex = filePath.indexOf(keyword);
            if (endIndex != -1) {
                //由于检测upload使得前面的uploadPath也没了，所以只去除/upload的位数
                filePath = filePath.substring(0, (filePath.length()-7));
            }

            System.out.println("filePath2  "+filePath);

            String deletaPath = filePath + StringUtils.substringAfter(oldPath, Constants.RESOURCE_PREFIX);
            System.out.println("5." + deletaPath);
            if (deleteFileRecursive(deletaPath)) {
                // 执行特定文件夹删除操作
                String fileExtension = getFileExtensionFromPath(deletaPath);
                if (fileExtension.equalsIgnoreCase("pdf")) {
                    String subFolderPath = getSubFolderPath(deletaPath, "pdf");
                    System.out.println("8." + subFolderPath);
                    deleteFolder(subFolderPath); // 修改这一行
                } else if (fileExtension.equalsIgnoreCase("doc") || fileExtension.equalsIgnoreCase("docx")) {
                    String subFolderPath = getSubFolderPath(deletaPath, "word");
                    System.out.println("9." + subFolderPath);
                    deleteFolder(subFolderPath); // 修改这一行
                }

                // 删除数据库中的文件记录
                int deleteCount = noteDataMapper.deleteNoteDataById(ids);
                System.out.println("文件删除成功");
                return true; // 文件删除成功
            } else {
                System.out.println("文件删除失败");
                return false; // 文件删除失败
            }
        }
        return false;
    }

    //只删除文件
    public boolean updateOnlyFile(Long ids) {
        // 根据ID查询文件信息
        String newFile = "";
        NoteData file = noteDataMapper.selectNoteDataById(ids);
        String oldFile = file.getPath();
        if (oldFile != null) {
            String filePath = RuoYiConfig.getProfile();
            String deletaPath = filePath + StringUtils.substringAfter(oldFile, Constants.RESOURCE_PREFIX);
            if (FileUtils.deleteFile(deletaPath)) {
                boolean flag = noteDataMapper.updateFileUploadUrlData(newFile, ids);
                if (flag) {
                    return true;
                }
            }
        }
        return false;
    }

    // 从文件路径中获取文件扩展名
    private String getFileExtensionFromPath(String filePath) {
        int dotIndex = filePath.lastIndexOf(".");
        if (dotIndex >= 0 && dotIndex < filePath.length() - 1) {
            return filePath.substring(dotIndex + 1).toLowerCase();
        }
        System.out.println("10." + dotIndex);
        return "";
    }

    // 根据文件路径和指定的文件夹名称获取子文件夹路径
    private String getSubFolderPath(String filePath, String folderName) {
        int index = filePath.lastIndexOf(File.separator + folderName + File.separator);
        if (index != -1) {
            return filePath.substring(0, index);
        }
        return filePath;
    }


    /**
     * 删除文件
     *
     * @param ids 文件
     * @return
     */
    public static boolean deleteFileRecursive(String ids) {
        File file = new File(ids);
        System.out.println("11." + file.getPath());
        if (file.isFile() && file.exists()) {
            return file.delete();
        }
        System.out.println(false);
        return false;
    }

    // 递归删除文件夹及其内容
    private void deleteFolder(String filePath) {
        File folder = new File(filePath);
        System.out.println("12." + folder);
        if (folder.exists()) {
            File[] files = folder.listFiles();
            if (files != null) {
                for (File file : files) {
                    if (file.isDirectory()) {
                        deleteFolder(file.getPath()); // 使用完整的文件夹路径
                    } else {
                        file.delete();
                    }
                }
                folder.delete();
            }
        }
    }

}
