package com.ruoyi.project.VRdemo.service.impl;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.project.VRdemo.domain.NoticeImData;
import com.ruoyi.project.VRdemo.domain.StudyImData;
import com.ruoyi.project.VRdemo.mapper.DataExcelMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.parameters.P;
import org.springframework.stereotype.Service;
import com.ruoyi.project.VRdemo.mapper.PressMapper;
import com.ruoyi.project.VRdemo.domain.Press;
import com.ruoyi.project.VRdemo.service.IPressService;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;

/**
 * pressService业务层处理
 * 
 * @author ruoyi
 * @date 2024-01-19
 */
@Service
public class PressServiceImpl implements IPressService 
{
    @Autowired
    private PressMapper pressMapper;

    @Autowired
    private DataExcelMapper dataExcelMapper;
    private static final Logger logger = LoggerFactory.getLogger(Press.class);

    private static final int BATCH_COUNT = 10000;
    /**
     * 查询press
     * 
     * @param pressId press主键
     * @return press
     */
    @Override
    public Press selectPressByPressId(Long pressId)
    {
        return pressMapper.selectPressByPressId(pressId);
    }

    /**
     * 查询press列表
     * 
     * @param press press
     * @return press
     */
    @Override
    public List<Press> selectPressList(Press press)
    {
        return pressMapper.selectPressList(press);
    }

    /**
     * 新增press
     * 
     * @param press press
     * @return 结果
     */
    @Override
    public int insertPress(Press press)
    {
        press.setCreateTime(DateUtils.getNowDate());
        return pressMapper.insertPress(press);
    }

    /**
     * 修改press
     * 
     * @param press press
     * @return 结果
     */
    @Override
    public int updatePress(Press press)
    {
        press.setUpdateTime(DateUtils.getNowDate());
        return pressMapper.updatePress(press);
    }

    /**
     * 批量删除press
     * 
     * @param pressIds 需要删除的press主键
     * @return 结果
     */
    @Override
    public int deletePressByPressIds(Long[] pressIds)
    {
        return pressMapper.deletePressByPressIds(pressIds);
    }

    /**
     * 删除press信息
     * 
     * @param pressId press主键
     * @return 结果
     */
    @Override
    public int deletePressByPressId(Long pressId)
    {
        return pressMapper.deletePressByPressId(pressId);
    }

    //EasyExcel批量导入
    @Override
    @Transactional(isolation = Isolation.READ_COMMITTED)
    public String importPress(List<Press> pressList, Boolean updateSupport) {
        if (StringUtils.isNull(pressList) || pressList.size() == 0) {
            throw new ServiceException("导入用户数据不能为空！");
        }

        List<Press> batchDataList = new ArrayList<>();
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();

        // 遍历用户数据列表，逐条处理数据
        for (Press press : pressList) {
            try {
                // 对每一条数据进行处理
                // 根据title字段判断数据合法性
                if (StringUtils.isEmpty(press.getTitle())) {
                    throw new ServiceException("模板标题不能为空！");
                }

                // 分开时间和刊期
                String dateAndIssue = press.getIssue();

                String dateStr = dateAndIssue.substring(0, dateAndIssue.indexOf("(")).trim();
                SimpleDateFormat ft = new SimpleDateFormat("yyyy");
                Date date = ft.parse(dateStr);

                String issue = dateAndIssue.substring(dateAndIssue.indexOf("(") + 1 ,dateAndIssue.indexOf(")")).trim();

                System.out.println(dateStr+"+++++"+issue);

                press.setDate(date);
                press.setIssue(issue);


                // 如果更新支持，检查数据库中是否存在相同的记录，如果存在，则进行更新操作
                // 判断数据库中是否存在相同标题的记录
                Press existingPress = null;
                if(updateSupport){
                    String titleAsString = press.getTitle();
                    existingPress= dataExcelMapper.selectPressByTitle(titleAsString);
                }
                if (existingPress != null) {
                    press.setUpdateBy(SecurityUtils.getLoginUser().getUsername());
                    press.setUpdateTime(DateUtils.getNowDate());
                    // 更新操作
                    dataExcelMapper.updatePressByImport(press);
                    successNum++;
                    successMsg.append("成功更新数据：").append(press.getTitle()).append("\n");
                } else {
                    // 检查是否已经存在于 batchDataList 中，避免重复插入
                    if (!batchDataList.contains(press)) {
                        press.setCreateBy(SecurityUtils.getLoginUser().getUsername());
                        press.setCreateTime(DateUtils.getNowDate());
                        batchDataList.add(press);
                    }
                    // 判断是否达到BATCH_COUNT，或者已经是最后一个数据
                    if (batchDataList.size() >= BATCH_COUNT || pressList.indexOf(press) == pressList.size() - 1) {
                        System.out.println( "batchDataList size: " + batchDataList);
                        // 执行批量插入操作
                        dataExcelMapper.insertPressDataList(batchDataList);
                        successNum += batchDataList.size();
                        for (Press insertedPress : batchDataList) {
                            successMsg.append("成功导入数据：").append(insertedPress.getTitle()).append("\n");
                        }
                        batchDataList.clear();
                    }
                }
            } catch (Exception e) {
                // 输出异常信息和上下文信息到日志
                logger.error("导入数据时发生异常：", e);
                logger.error("异常发生时的数据：{}", press); // 这里的 study 是导致异常的数据对象
                // 处理导入失败的情况
                failureNum++;
                failureMsg.append("导入失败，模板标题为：").append(press.getTitle()).append("\n");
            }
        }
        // 判断是否有导入失败的数据
        if (failureNum > 0) {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确或插入数据库出错，错误如下：\n");
            throw new ServiceException(failureMsg.toString());
        } else {
            // 若全部数据导入成功，则返回成功导入的信息
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：\n");
        }
        return successMsg.toString();
    }
}
