package com.ruoyi.project.VRdemo.service.impl;

import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.constant.UserConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.FileUtils;
import com.ruoyi.framework.config.RuoYiConfig;
import com.ruoyi.project.VRdemo.domain.NoteNationData;
import com.ruoyi.project.VRdemo.domain.NoticeImData;
import com.ruoyi.project.VRdemo.domain.StudyData;
import com.ruoyi.project.VRdemo.domain.StudyImData;
import com.ruoyi.project.VRdemo.domain.vo.Catalog;
import com.ruoyi.project.VRdemo.domain.vo.StudyDataVo;
import com.ruoyi.project.VRdemo.mapper.DataExcelMapper;
import com.ruoyi.project.VRdemo.mapper.StudyDataMapper;
import com.ruoyi.project.VRdemo.service.IStudyDataService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

@Service
public class StudyDataServiceImpl implements IStudyDataService {
    @Autowired
    private StudyDataMapper studyDataMapper;
    @Autowired
    private DataExcelMapper dataExcelMapper;

    private static final int BATCH_COUNT = 100;
    private static final Logger logger = LoggerFactory.getLogger(NoticeImData.class);

    /**
     * 查询study
     *
     * @param id study主键
     * @return study
     */
    @Override
    public StudyData selectStudyDataById(Long id) {
        return studyDataMapper.selectStudyDataById(id);
    }

    /**
     * 查询study列表
     *
     * @param studyData study
     * @return study
     */
    @Override
    public List<StudyData> selectStudyDataList(StudyData studyData) {
        return studyDataMapper.selectStudyDataList(studyData);
    }

    /**
     * 新增study
     *
     * @param studyData study
     * @return 结果
     */
    @Override
    public int insertStudyData(StudyData studyData) {
        studyData.setCreateTime(DateUtils.getNowDate());
        return studyDataMapper.insertStudyData(studyData);
    }

    /**
     * 校验用户名称是否唯一
     *
     * @param studyData 用户信息
     * @return 结果
     */
    @Override
    public boolean checkUserNameUnique(StudyData studyData) {
        Long userId = StringUtils.isNull(studyData.getId()) ? -1L : studyData.getId();
        StudyData info = studyDataMapper.checkUserNameUnique(studyData.getTitle());
        if (StringUtils.isNotNull(info) && info.getId().longValue() != userId.longValue()) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 修改study
     *
     * @param studyData study
     * @return 结果
     */
    @Override
    public int updateStudyData(StudyData studyData) {
        studyData.setUpdateTime(DateUtils.getNowDate());
        return studyDataMapper.updateStudyData(studyData);
    }

    /**
     * 批量删除study
     *
     * @param ids 需要删除的study主键
     * @return 结果
     */
    @Override
    public int deleteStudyDataByIds(Long[] ids) {
        return studyDataMapper.deleteStudyDataByIds(ids);
    }

    /**
     * 删除study信息
     *
     * @param id study主键
     * @return 结果
     */
    @Override
    public int deleteStudyDataById(Long id) {
        return studyDataMapper.deleteStudyDataById(id);
    }

    @Override
    public List<StudyData> selectStudyDataListVo(StudyDataVo studyDataVo) {
        return studyDataMapper.selectStudyDataListVo(studyDataVo);
    }

    @Override
    public List<StudyData> selectStudyDataListVoAll(StudyDataVo studyDataVo) {
        return studyDataMapper.selectStudyDataListVoAll(studyDataVo);
    }

    @Override
    public List<Catalog> selectCatalog(Catalog catalog) {
        return studyDataMapper.selectCatalog(catalog);
    }

    //删除
    public boolean deleteFile(Long ids) {
        // 根据ID查询文件信息
        StudyData file = studyDataMapper.selectStudyDataById(ids);
        System.out.println("4." + file);
        if (file == null) {
            return false; // 文件不存在，删除失败
        }

        String oldPath = file.getPath();
        if (oldPath != null) {
            String filePath = RuoYiConfig.getUploadPath();
            //由于在RuoYiConfig.getUploadPath()中带有upload导致文件搜索失败，所以去除
            String keyword = "/upload";
            int endIndex = filePath.indexOf(keyword);
            if (endIndex != -1) {
                filePath = filePath.substring(0, (filePath.length()-7));
            }
            String deletaPath = filePath + StringUtils.substringAfter(oldPath, Constants.RESOURCE_PREFIX);
            System.out.println("5." + deletaPath);
            if (deleteFileRecursive(deletaPath)) {
                // 执行特定文件夹删除操作
                String fileExtension = getFileExtensionFromPath(deletaPath);
                if (fileExtension.equalsIgnoreCase("pdf")) {
                    String subFolderPath = getSubFolderPath(deletaPath, "pdf");
                    System.out.println("8." + subFolderPath);
                    deleteFolder(subFolderPath); // 修改这一行
                } else if (fileExtension.equalsIgnoreCase("doc") || fileExtension.equalsIgnoreCase("docx")) {
                    String subFolderPath = getSubFolderPath(deletaPath, "word");
                    System.out.println("9." + subFolderPath);
                    deleteFolder(subFolderPath); // 修改这一行
                }

                // 删除数据库中的文件记录
                int deleteCount = studyDataMapper.deleteStudyDataById(ids);
                System.out.println("文件删除成功");
                return true; // 文件删除成功
            } else {
                System.out.println("文件删除失败");
                return false; // 文件删除失败
            }
        }
        return false;
    }

    //只删除文件
    public boolean updateOnlyFile(Long ids) {
        // 根据ID查询文件信息
        String newFile = "";
        StudyData file = studyDataMapper.selectStudyDataById(ids);
        String oldFile = file.getPath();
        if (oldFile != null) {
            String filePath = RuoYiConfig.getProfile();
            String deletaPath = filePath + StringUtils.substringAfter(oldFile, Constants.RESOURCE_PREFIX);
            if (FileUtils.deleteFile(deletaPath)) {
                boolean flag = studyDataMapper.updateFileUploadUrlData(newFile, ids);
                if (flag) {
                    return true;
                }
            }
        }
        return false;
    }

    // 从文件路径中获取文件扩展名
    private String getFileExtensionFromPath(String filePath) {
        int dotIndex = filePath.lastIndexOf(".");
        if (dotIndex >= 0 && dotIndex < filePath.length() - 1) {
            return filePath.substring(dotIndex + 1).toLowerCase();
        }
        System.out.println("10." + dotIndex);
        return "";
    }

    // 根据文件路径和指定的文件夹名称获取子文件夹路径
    private String getSubFolderPath(String filePath, String folderName) {
        int index = filePath.lastIndexOf(File.separator + folderName + File.separator);
        if (index != -1) {
            return filePath.substring(0, index);
        }
        return filePath;
    }


    /**
     * 删除文件
     *
     * @param ids 文件
     * @return
     */
    public static boolean deleteFileRecursive(String ids) {
        File file = new File(ids);
        System.out.println("11." + file.getPath());
        if (file.isFile() && file.exists()) {
            return file.delete();
        }
        System.out.println(false);
        return false;
    }

    // 递归删除文件夹及其内容
    private void deleteFolder(String filePath) {
        File folder = new File(filePath);
        System.out.println("12." + folder);
        if (folder.exists()) {
            File[] files = folder.listFiles();
            if (files != null) {
                for (File file : files) {
                    if (file.isDirectory()) {
                        deleteFolder(file.getPath()); // 使用完整的文件夹路径
                    } else {
                        file.delete();
                    }
                }
                folder.delete();
            }
        }
    }

    //EasyExcel批量导入
    @Override
    @Transactional(isolation = Isolation.READ_COMMITTED)
    public String importStudy(List<StudyImData> studyList, Boolean updateSupport) {
        if (StringUtils.isNull(studyList) || studyList.size() == 0) {
            throw new ServiceException("导入用户数据不能为空！");
        }
        List<StudyImData> batchDataList = new ArrayList<>();
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        // 遍历用户数据列表，逐条处理数据
        for (StudyImData study : studyList) {
            try {
                // 对每一条数据进行处理
                // 根据title字段判断数据合法性
                if (StringUtils.isEmpty(study.getTitle())) {
                    throw new ServiceException("模板标题不能为空！");
                }
                // 如果更新支持，检查数据库中是否存在相同的记录，如果存在，则进行更新操作
                // 判断数据库中是否存在相同标题的记录
                String titleAsString = study.getTitle();
                StudyImData existingStudy = dataExcelMapper.selectStudyByTitle(titleAsString);

                // 根据标签字段查询对应的字典值
                String yearLabel = study.getYear(); // 获取年份标签
                String monthLabel = study.getMonth(); // 获取月份标签
                String yearValue = dataExcelMapper.findValueByLabel("year_type", yearLabel); // 查询年份值
                String monthValue = dataExcelMapper.findValueByLabel("month_type", monthLabel); // 查询月份值

                // 设置字典值到StudyImData对象
                study.setYear(yearValue);
                study.setMonth(monthValue);

                if (existingStudy != null && updateSupport) {
                    // 更新操作
                    dataExcelMapper.updateImStudy(study);
                    successNum++;
                    successMsg.append("成功更新数据：").append(study.getTitle()).append("\n");
                } else {
                    // 设置企业ID和数据类型默认值
                    study.setCompanyId(3L);
                    study.setDataType(2L);
                    // 检查是否已经存在于 batchDataList 中，避免重复插入
                    if (!batchDataList.contains(study)) {
                        batchDataList.add(study);
                    }
                    // 判断是否达到BATCH_COUNT，或者已经是最后一个数据
                    if (batchDataList.size() >= BATCH_COUNT || studyList.indexOf(study) == studyList.size() - 1) {
                        // 执行批量插入操作
                        List<StudyImData> insertList = new ArrayList<>(batchDataList); // 复制列表
                        dataExcelMapper.insertStudyDataList(insertList); // 使用复制的列表进行插入
                        successNum += insertList.size();
                        for (StudyImData insertedStudy : insertList) {
                            successMsg.append("成功导入数据：").append(insertedStudy.getTitle()).append("\n");
                        }
                        batchDataList.clear();


                    }
                }
            } catch (Exception e) {
                // 输出异常信息和上下文信息到日志
                logger.error("导入数据时发生异常：", e);
                logger.error("异常发生时的数据：{}", study); // 这里的 study 是导致异常的数据对象
                // 处理导入失败的情况
                failureNum++;
                failureMsg.append("导入失败，模板标题为：").append(study.getTitle()).append("\n");
            }
        }
        // 判断是否有导入失败的数据
        if (failureNum > 0) {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确或插入数据库出错，错误如下：\n");
            throw new ServiceException(failureMsg.toString());
        } else {
            // 若全部数据导入成功，则返回成功导入的信息
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：\n");

        }
        return successMsg.toString();
    }
}
